/************************************************************
    Program:  CheckSyntax.p
    Created:  4 Jun 1999
     Author:  Tim Townsend
Description:  Check syntax from within Ed4W V4

************************************************************/

def var cFile                as char                       no-undo.
def var cErrFile             as char                       no-undo.
def var cErrMsg              as char                       no-undo.
def var i                    as int                        no-undo.
def var lCompileOnSave       as log   init false           no-undo.
def var cDLC                 as char                       no-undo.
def var iEd4W                as int                        no-undo.

def temp-table ttMatch no-undo
  field MatchName as char
  field MatchChars as int
  index Main is primary unique MatchName
  index MatchChars MatchChars.

def stream sSource.
def frame fDDE
  with no-labels overlay three-d.

assign frame fDDE:hidden = true
       default-window:window-state = window-minimized.

function Dot-R-Exists returns logical
         (pSourceFile as char):
  return search(substr(pSourceFile,1,r-index(pSourceFile,".")) + "r":u) <> ?.
end function.

function ParseName return char
         (pCodeFragment as char,
          pFromRight as logical):
def var cCharsIllegalInName  as char                       no-undo
  init " ,.()[]'+-*/^".
def var cTempLine            as char                       no-undo.
def var i                    as int                        no-undo.

  cTempLine = pCodeFragment.
  do i = 1 to length(cCharsIllegalInName):
    cTempLine = replace(cTempLine,substr(cCharsIllegalInName,i,1)," ").
  end.  /* 1..? */

  if pFromRight then
    return entry(num-entries(cTempLine," "),cTempLine," ").
  else
    return entry(1,cTempLine," ").
end function.

pause 0 before-hide.

assign
  cFile     = entry(1,session:parameter)
  cDLC      = search("bin/prowin32.exe":u)
  cDLC      = substr(cDLC,1,length(cDLC) - length("bin/prowin32.exe":u))
  &if decimal(substr(proversion,1,index(proversion,".") + 1)) >= 9.0 &then
    session:debug-alert = false
  &endif
  .

if num-dbs > 0 then do:
  create alias dictdb2    for database value(ldbname(1)) no-error.
end.

propath = propath + "," + cDLC + "src".

if cFile > "" then
do:
  run CheckCompileFlag(cFile,output lCompileOnSave).
  lCompileOnSave = lCompileOnSave or Dot-R-Exists(cFile).
  compile value(cFile)
    save = lCompileOnSave
    no-error.
  /* errors */
  if compiler:error then do:
    do i = 1 to error-status:num-messages:
      cErrMsg = cErrMsg + error-status:get-message(i) + "~r".
    end.  /* 1..? */
/* Uncomment this to have the propath appended to error message
    cErrMsg = cErrMsg + "Propath:" + "~r".
    do i = 1 to num-entries(propath):
      cErrMsg = cErrMsg + "  " + entry(i,propath) + "~r".
    end.  /* 1..? */
*/
    assign file-info:file-name = search(compiler:file-name)
           cErrFile            = file-info:full-pathname.
    assign cErrFile            = cFile when cErrFile = ?.

    run WriteToEd(cErrMsg,true).

    case error-status:get-number(1):
      when 201 /* unknown field or var */ then
        run GuessFieldName.
      when 725 /* unknown table */ or when 200 then
        run GuessTableName.
    end case.

  end.
  /* warnings */
  else if error-status:get-message(1) > "" then do:
    do i = 1 to error-status:num-messages:
      cErrMsg = cErrMsg + error-status:get-message(i) + "~r".
    end.  /* 1..? */

    run WriteToEd("Program compiled, but..." + "~r" + cErrMsg,true).

  end.
  /* OK */
  else do:
    message
      "Syntax is correct." +
      (if lCompileOnSave then "~r.r created." else "")
      view-as alert-box info button ok title "Ed4W Check Syntax".
/*
    run WriteToEd("Syntax is correct." +
                  (if lCompileOnSave then "~r.r created." else ""),
                  false).
*/
  end.
end.  /* if */

if program-name(2) = ? then
  quit.

/****************************************************************************/
procedure CheckCompileFlag:
/*--------------------------------------------------------------------------
  This procedure checks for the existance of the "compile on save"
  flag (set in the UIB procedure settings dialog) in the source code.
--------------------------------------------------------------------------*/
def input        param cFileName            as char        no-undo.
def       output param lCompileOnSave       as log         no-undo.

def var cCodeLine            as char                       no-undo.
def var lProcSettingsSect    as log                        no-undo.

  input stream sSource from value(cFileName).
  repeat:
    import stream sSource unformatted cCodeLine.
    cCodeLine = trim(cCodeLine).
    if cCodeLine = "&ANALYZE-SUSPEND _PROCEDURE-SETTINGS" then
      lProcSettingsSect = true.
    else if lProcSettingsSect and
            cCodeLine begins "&ANALYZE-RESUME" then
      lProcSettingsSect = false.
    else if lProcSettingsSect and
            cCodeLine begins "Other Settings:" then do:
      assign lCompileOnSave = (index(cCodeLine + " "," COMPILE ") > 0).
      leave.  /* found it, we're done */
    end.  /* else if */
  end.  /* repeat */
  input stream sSource close.

  return.
end procedure.

/****************************************************************************/
procedure WriteToEd:
/*--------------------------------------------------------------------------
  This procedure writes text to the Ed4W Debug window.
--------------------------------------------------------------------------*/
def input        param pMsg                 as char        no-undo.
def input        param pError               as log         no-undo.

def var cEdMsg               as char                       no-undo.
def var iLn                  as int                        no-undo.

  /* replace invalid characters */
  assign
    cEdMsg = replace(pMsg,'"',"'").

  /* start dde conversation */
  view frame fDDE.
  dde initiate iEd4W frame frame fDDE:handle
          application "Ed32" topic "System".

  /* send each line */
  do iLn = 1 to num-entries(cEdMsg,"~r"):
    dde execute iEd4W command
          '[open(""-k "K_ActivateOutput,param=1,' +
          (if iLn = 1 then 'K_SelectAll,K_Delete,'
           else 'K_LineSplit,') +
          '\"' +
          entry(iLn,cEdMsg,"~r") +
          '\"")]'.
  end.  /* 1..? */

  /* error tracking type = Progress, position to top line, first error */
  if pError then
    dde execute  iEd4W
      command '[open("" -k "K_ActivateOutput,param=1,K_BuildErrorType,param=Progress,K_DocumentStart,K_GoToNextError")]'.

  /* close dde conversation */
  dde terminate iEd4W.
  hide frame fDDE no-pause.
  
  return.
end procedure.

/****************************************************************************/
procedure GuessFieldName:
/*--------------------------------------------------------------------------
  This procedure tries to guess at the intended field name.
  Error 201 has this format:
    ** Unknown Field or Variable name - xxxxx. (201)
--------------------------------------------------------------------------*/
def var cCodeFldName         as char                       no-undo.
def var cCodeLine            as char                       no-undo.
def var cErrLine             as char                       no-undo.
def var cFldList             as char                       no-undo.
def var cFldName             as char                       no-undo.
def var cLineFirstPart       as char                       no-undo.
def var cNewFldName          as char                       no-undo.
def var cTableName           as char                       no-undo.
def var iFldPos              as int                        no-undo.
def var iLeadMatchChars      as int                        no-undo.
def var iLen                 as int                        no-undo.
def var iPos                 as int                        no-undo.
def var iTotMatchChars       as int                        no-undo.
def var iTrailMatchChars     as int                        no-undo.

  run GetCurrentLine(output cCodeLine).
  assign cErrLine     = error-status:get-message(1)
         cFldName     = left-trim(entry(1,substr(cErrLine,index(cErrLine,"-")),"."),"- ")
         iFldPos      = compiler:error-column
         cCodeFldName = ParseName(substr(cCodeLine,iFldPos),no).

  if cCodeFldName begins cFldName then
    cCodeFldName = cFldName.
           
  if cFldName > "" and cFldName = cCodeFldName then do:
    assign cLineFirstPart = right-trim(substr(cCodeLine,1,compiler:error-column - 1),".")
           cTableName     = ParseName(cLineFirstPart,yes).

    if cTableName > "" then do:
      assign iLen       = length(cTableName).

      for first _file where _file._file-name = cTableName no-lock,
          each _field of _file no-lock:

        assign iLeadMatchChars  = 0
               iTrailMatchChars = 0.

        do while substr(_field._field-name,1,iLeadMatchChars + 1) = substr(cFldName,1,iLeadMatchChars + 1) and
                 iLeadMatchChars < min(length(_field._field-name),length(cFldName)):
          iLeadMatchChars = iLeadMatchChars + 1.
        end.  /* do while */
        do while substr(_field._field-name,length(_field._field-name) - iTrailMatchChars,iTrailMatchChars + 1) =
                 substr(cFldName,length(cFldName) - iTrailMatchChars,iTrailMatchChars + 1) and
                 iTrailMatchChars < min(length(_field._field-name),length(cFldName)):
          iTrailMatchChars = iTrailMatchChars + 1.
        end.  /* do while */
        iTotMatchChars = iLeadMatchChars + iTrailMatchChars.

        if iTotMatchChars > 0 then do:
          create ttMatch.
          assign ttMatch.MatchName    = _field._field-name
                 ttMatch.MatchChars = iTotMatchChars.
        end.  /* if */
      end.  /* for each */

      for each ttMatch
          where ttMatch.MatchChars > 1
          by ttMatch.MatchChars descending:
        cFldList = cFldList + ttMatch.MatchName + ",".
      end.  /* for each */

      if cFldList > "" then do:
        run PresentGuessList(trim(cFldList,","),output cNewFldName).
        if cNewFldName > "" then do:
          substr(cCodeLine,iFldPos,length(cFldName)) = cNewFldName.
          run PasteToEd(cCodeLine).
        end.
      end.  /* if */

    end.  /* if */
  end.  /* if */

  return.
end procedure.

/****************************************************************************/
procedure GuessTableName:
/*--------------------------------------------------------------------------
  This procedure tries to guess at the intended table name.
  Error 725 has this format:
  Unknown or ambiguous table xxx. (725)

  Error 200 has this format:
  ** Unknown table name xxx. (200)

--------------------------------------------------------------------------*/
def var cCodeTblName         as char                       no-undo.
def var cCodeLine            as char                       no-undo.
def var cErrLine             as char                       no-undo.
def var cTblList             as char                       no-undo.
def var cTblName             as char                       no-undo.
def var cLineFirstPart       as char                       no-undo.
def var cNewTblName          as char                       no-undo.
def var iTblPos              as int                        no-undo.
def var iLeadMatchChars      as int                        no-undo.
def var iLen                 as int                        no-undo.
def var iPos                 as int                        no-undo.
def var iTotMatchChars       as int                        no-undo.
def var iTrailMatchChars     as int                        no-undo.

  run GetCurrentLine(output cCodeLine).
  assign cErrLine     = error-status:get-message(1)
         iTblPos      = compiler:error-column
         cCodeTblName = entry(1,substr(cCodeLine,iTblPos)," ")
         cTblName     = entry(1,cErrLine,".")
         cTblName     = entry(num-entries(cTblName," "),cTblName," ")
         iTblPos      = index(cCodeLine,cTblName) no-error.

  if cTblName > "" and (cTblName = cCodeTblName or iTblPos > 0) then do:
    assign cLineFirstPart = substr(cCodeLine,1,compiler:error-column - 1)
           iPos           = r-index(cLineFirstPart," ") + 1.

    if cTblName > "" then do:

      for each _file no-lock:

        assign iLeadMatchChars  = 0
               iTrailMatchChars = 0.

        do while substr(_file._file-name,1,iLeadMatchChars + 1) = substr(cTblName,1,iLeadMatchChars + 1) and
                 iLeadMatchChars < min(length(_file._file-name),length(cTblName)):
          iLeadMatchChars = iLeadMatchChars + 1.
        end.  /* do while */
        do while substr(_file._file-name,length(_file._file-name) - iTrailMatchChars,iTrailMatchChars + 1) =
                 substr(cTblName,length(cTblName) - iTrailMatchChars,iTrailMatchChars + 1) and
                 iTrailMatchChars < min(length(_file._file-name),length(cTblName)):
          iTrailMatchChars = iTrailMatchChars + 1.
        end.  /* do while */
        iTotMatchChars = iLeadMatchChars + iTrailMatchChars.

        if iTotMatchChars > 0 then do:
          create ttMatch.
          assign ttMatch.MatchName  = _file._file-name
                 ttMatch.MatchChars = iTotMatchChars.
        end.  /* if */
      end.  /* for each */

      for each ttMatch
          where ttMatch.MatchChars > int(length(cTblName) * 0.333333)
          by ttMatch.MatchChars descending:
        cTblList = cTblList + ttMatch.MatchName + ",".
      end.  /* for each */

      if cTblList > "" then do:
        run PresentGuessList(trim(cTblList,","),output cNewTblName).
        if cNewTblName > "" then do:
          substr(cCodeLine,iTblPos,length(cTblName)) = cNewTblName.
          run PasteToEd(cCodeLine).
        end.
      end.  /* if */

    end.  /* if */
  end.  /* if */

  return.
end procedure.

/****************************************************************************/
procedure GetCurrentLine:
/*--------------------------------------------------------------------------
  This procedure reads the current line in the document.
--------------------------------------------------------------------------*/
def       output param pText                as char        no-undo.

def var iEd4W                as int                        no-undo.

  /* start dde conversation */
  view frame fDDE.
  dde initiate iEd4W frame frame fDDE:handle
          application "Ed32" topic "System".

  dde execute iEd4W command
        '[open(""-k "K_SelectLine,K_Copy""")]'.
  pText = trim(clipboard:value,"~r~n":u).

  /* close dde conversation */
  dde terminate iEd4W.
  hide frame fDDE no-pause.

  return.
end procedure. /* GetCurrentLine */

/****************************************************************************/
procedure PasteToEd:
/*--------------------------------------------------------------------------
  This procedure writes text to the current line in the document.
--------------------------------------------------------------------------*/
def input        param pText                as char        no-undo.

def var iEd4W                as int                        no-undo.

  /* start dde conversation */
  view frame fDDE.
  dde initiate iEd4W frame frame fDDE:handle
          application "Ed32" topic "System".

  /* send text */
  clipboard:value = pText + "~r~n".
  dde execute iEd4W command
        '[open(""-k "K_SelectLine,K_Paste""")]'.

  /* close dde conversation */
  dde terminate iEd4W.
  hide frame fDDE no-pause.
  
  return.
end procedure.

/****************************************************************************/
procedure PresentGuessList:
/*--------------------------------------------------------------------------
  This procedure presents a list of guesses
--------------------------------------------------------------------------*/
def input        param pcGuessList          as char        no-undo.
def       output param cChoice              as char        no-undo.

def var cGuessList             as char                       no-undo
  view-as selection-list inner-lines 10 inner-chars 30 scrollbar-vertical.
def button btnYes label "Yes" auto-go.
def button btnNo label "No" auto-endkey.

def frame fGuess
  skip(.5)
  space(1) cGuessList no-label space(1)
  skip(.5)
  btnYes at 10 btnNo
  skip(.5)
    with view-as dialog-box three-d overlay no-labels title "Did you mean..."
         default-button btnYes.

  on default-action of cGuessList in frame fGuess
    apply "go" to frame fGuess.

  do with frame fGuess on endkey undo, leave:
    cGuessList:list-items = pcGuessList.
    cGuessList:load-mouse-pointer("glove").
    view frame fGuess.
    cGuessList:screen-value = cGuessList:entry(1) no-error.
    enable all.
    wait-for go of frame fGuess.
    cChoice = cGuessList:screen-value.
  end.

  return.
end procedure.


