/************************************************************
    Program:  Expand-Of.p
    Created:  20 Mar 2003
     Author:  Tim Townsend
Description:  Expand an 'of' reference inside Ed4W V4

************************************************************/

def var cText                as char                       no-undo.
def var lChanged             as log                        no-undo.

def temp-table ttIndex no-undo
  field Table1IndexName as char
  field Table2IndexName as char
  field FieldList       as char
  field IsPrimary       as logical
  field IsUnique        as logical
  index Main is primary unique Table1IndexName Table2IndexName.

def frame fDDE
  with no-labels overlay three-d.

assign frame fDDE:hidden = true
       default-window:window-state = window-minimized.


run GetCurrentLine(output cText).
run ExpandOf(input-output cText,output lChanged).
if lChanged then
  run WriteToEd(cText).

if program-name(2) = ? then quit.


/****************************************************************************/
procedure WriteToEd:
/*--------------------------------------------------------------------------
  This procedure writes text to the current line in the document.
--------------------------------------------------------------------------*/
def input        param pText                as char        no-undo.

def var iEd4W                as int                        no-undo.

  /* start dde conversation */
  view frame fDDE.
  dde initiate iEd4W frame frame fDDE:handle
          application "Ed32" topic "System".

  /* send text */
  clipboard:value = pText + "~r~n".
  dde execute iEd4W command
        '[open(""-k "K_SelectLine,K_Paste""")]'.

  /* close dde conversation */
  dde terminate iEd4W.
  hide frame fDDE no-pause.
  
  return.
end procedure.

/****************************************************************************/
procedure GetCurrentLine:
/*--------------------------------------------------------------------------
  This procedure reads the current line in the document.
--------------------------------------------------------------------------*/
def       output param pText                as char        no-undo.

def var iEd4W                as int                        no-undo.

  /* start dde conversation */
  view frame fDDE.
  dde initiate iEd4W frame frame fDDE:handle
          application "Ed32" topic "System".

  dde execute iEd4W command
        '[open(""-k "K_SelectLine,K_Copy""")]'.
  pText = clipboard:value.

  /* close dde conversation */
  dde terminate iEd4W.
  hide frame fDDE no-pause.

  return.
end procedure. /* GetCurrentLine */

/****************************************************************************/
procedure ExpandOf:
def input-output param pText                as char        no-undo.
def       output param pChanged             as log         no-undo.

def var cFieldList           as char                       no-undo.
def var cSrcFieldList        as char                       no-undo.
def var cSrcTableName        as char                       no-undo.
def var cTableName           as char                       no-undo.
def var cTermChar            as char                       no-undo.
def var cWherePhrase         as char                       no-undo.
def var i                    as int                        no-undo.
def var iOfPos               as int                        no-undo.
def var iSrcTablePos         as int                        no-undo.
def var iTablePos            as int                        no-undo.
def var cFieldName           as char                       no-undo.
def var cIndexFieldList      as char                       no-undo.
def var lUnique              as log                        no-undo.
def var lPrimary             as log                        no-undo.

  def buffer bSrcFile        for _file.
  def buffer bSrcField       for _field.
  def buffer bSrcIndex       for _index.

  assign iOfPos = lookup("of",pText," ").

  if iOfPos > 1 then do:
    /* parse out table from line of text*/
    assign iTablePos     = iOfPos - 1
           cTableName    = entry(iTablePos,pText," ")
           iSrcTablePos  = iOfPos + 1
           cSrcTableName = trim(entry(iSrcTablePos,pText + " "," ")).

    /* separate terminating character if there is one */
    if cTableName > "" and cSrcTableName > "" then do:
      if index(".:",substr(cSrcTableName,length(cSrcTableName))) > 0 then
        assign cTermChar     = substr(cSrcTableName,length(cSrcTableName))
               cSrcTableName = substr(cSrcTableName,1,length(cSrcTableName) - 1).
    end.  /* if */

    /* find matching index fields */
    IndexLoop:
    for first _file
        where _file._file-name = cTableName
        no-lock,
        each _index of _file
        no-lock
        by _index._unique descending
        by _index._num-comp:

      assign cFieldList = ""
             cTableName = _file._file-name.
      for each _index-field of _index no-lock,
          first _field of _index-field no-lock
          by _index-field._index-seq:
        cFieldList = trim(cFieldList + "," + _field._field-name,",").
      end.  /* for each */

      for first bSrcFile
          where bSrcFile._file-name = cSrcTableName
          no-lock,
          each bSrcIndex of bSrcFile
          no-lock
          by bSrcIndex._unique descending
          by bSrcIndex._num-comp:

        assign cSrcFieldList = ""
               cSrcTableName = bSrcFile._file-name.
        for each _index-field of bSrcIndex no-lock,
            first _field of _index-field no-lock
            by _index-field._index-seq:
          cSrcFieldList = trim(cSrcFieldList + "," + _field._field-name,",").
        end.  /* for each */

        assign
          lUnique = (_index._unique or bSrcIndex._unique)
          lPrimary = (recid(_index) = _file._prime-index or recid(bSrcIndex) = bSrcFile._prime-index).

        if cFieldList begins cSrcFieldList or
           cSrcFieldList begins cFieldList or
           cFieldList = cSrcFieldList then do:
          create ttIndex.
          assign
            ttIndex.Table1IndexName = _index._index-name
            ttIndex.Table2IndexName = bSrcIndex._index-name
            ttIndex.FieldList       = (if num-entries(cFieldList) < num-entries(cSrcFieldList)
                                       then cFieldList else cSrcFieldList)
            ttIndex.IsPrimary       = lPrimary
            ttIndex.IsUnique        = lUnique.
        end.  /* if */

      end.  /* for first,each */
      cFieldList = "".
    end.  /* for first,each */

    /* determine which index to use */
    for each ttIndex
        by ttIndex.IsPrimary descending
        by ttIndex.IsUnique descending
        by num-entries(ttIndex.FieldList) descending:
      cIndexFieldList = ttIndex.FieldList.
      leave.
    end.  /* for each */

    /* build where phrase */
    if cIndexFieldList > "" then do:
      assign cWherePhrase = "where"
             pChanged     = true.
      do i = 1 to num-entries(cIndexFieldList):
        assign
          cFieldName = entry(i,cIndexFieldList)
          cWherePhrase = cWherePhrase  + " " +
                         (if i > 1 then "and " else "") +
                         cTableName    + "." + cFieldName + " = " +
                         cSrcTableName + "." + cFieldName.
      end.  /* 1..? */

      assign
        entry(iSrcTablePos,pText," ") = ""
        entry(iOfPos,pText," ")       = cWherePhrase + cTermChar.
    end.  /* if */

  end.  /* if */

  return.
end procedure. /* ExpandOf */
